#!/bin/sh
#	Start and Stop script
#
# This file will enable the user to start and stop the PowerPack for Web Servers
# Agent in Unix Environment.
#
# CA (tm) Introscope(R) PowerPack(tm) for Web Servers                    	
#																					 
#																					 
# CA Wily Introscope(R) Version 9.7.0 Build 27									 
# Copyright (c) 2014 CA. All Rights Reserved.									 
# Introscope(R) is a registered trademark of CA.		
# 
########################################################################
# define HOSTNAME and WSAGENT_PID_FILE_NAME
HOSTNAME_CMD=hostname
readonly HOSTNAME_CMD
SCRIPT_DIR=

## For SSL, the IBM Websphere MQ 6.0.0.0 JRE does not work. Use Sun JRE v1.4 onwards or latest IBM Websphere MQ JRE
JAVA_HOME=/usr

## For MQ 6.x , MQ 7.x , MB 6.x jars
MQ_JAR=./lib/com.ibm.mq.jar
PCF_JAR=./lib/com.ibm.mq.pcf6-1.jar
CONNECTOR_JAR=./lib/connector.jar
MQJMS_JAR=./lib/com.ibm.mqjms.jar
DHBCORE_JAR=./lib/dhbcore.jar
J2EE_JAR=./lib/j2ee.jar

##  For MQ 7.x, MB 7.0 jars uncomment following lines.
MQHEADERS_JAR=./lib/com.ibm.mq.headers.jar
MQCOMMONSERVICES_JAR=./lib/com.ibm.mq.commonservices.jar
MQJMQI_JAR=./lib/com.ibm.mq.jmqi.jar

##  For com.ibm.mq.pcf.jar obtained from the WebSphere MQ v7.0 installation path
PCF_JAR_7=./lib/com.ibm.mq.pcf.jar

## For MB 6.0/6.1/7.0
CONFIGMANAGERPROXY_JAR=./lib/ConfigManagerProxy.jar

## For MB 8.0
IBMJSEE_PROVIDER2=./lib/ibmjsseprovider2.jar

CLASSPATH=MQMonitor.jar:./lib/Agent.jar:./lib/jline-0.9.94.jar:$PCF_JAR:$PCF_JAR_7:$MQ_JAR:$CONNECTOR_JAR:$CONFIGMANAGERPROXY_JAR:$MQJMS_JAR:$DHBCORE_JAR:$J2EE_JAR:$MQHEADERS_JAR:$MQCOMMONSERVICES_JAR:$MQJMQI_JAR:$IBMJSEE_PROVIDER2:.


# check that hostname is working
if which "$HOSTNAME_CMD" >/dev/null 2>&1; then
	if HOSTNAME=`$HOSTNAME_CMD` >/dev/null 2>&1; then
		if [ -z $HOSTNAME ] ; then
			echo "ERROR: Unable to get the host name of this server using the command \"$HOSTNAME_CMD\""
			exit 1
		fi
		#echo "DEBUG: hostname=\"${HOSTNAME}\""
	else
		echo "ERROR: Unable to get the host name of this server using the command \"$HOSTNAME_CMD\""
		exit 1
	fi
else
	echo "ERROR: Executable \"$HOSTNAME_CMD\" not found in \$PATH"
	exit 1
fi
readonly HOSTNAME
                           
MQAGENT_PID_FILE_NAME=./pids/MQA.${HOSTNAME}.pid
readonly MQAGENT_PID_FILE_NAME


# attempt to guess in which directory this script and go there
# changes to this directory if necessary
# takes one parameter which is the name of the script
gotoScriptDir() {
	if [ "$#" != "1" ] ; then
		echo "ERROR: Function gotoScriptDir takes 1 argument"
		exit 1
	fi

	SCRIPT_PATH="$1"
	# echo "DEBUG: SCRIPT_PATH1=$SCRIPT_PATH"

	if [ ! -f "$SCRIPT_PATH" ] ; then
		# probably means that the script is in the $PATH
		# let's explore the path and see if we can find it
 		case "$SCRIPT_PATH" in
			/*)
				# absolute path - do not search PATH
				echo "ERROR: unable to find directory that contains this script" 
	    			exit 1
				;;

			*)
				# Search all directories from the PATH variable. Take
				# care to interpret leading and trailing ":" as meaning
				# the current directory; the same is true for "::" within
				# the PATH.
				
				for DIR in `echo "$PATH" | sed 's/^:/.:/g;s/::/:.:/g;s/:$/:./;s/:/ /g'` ; do
					if [ -f "$DIR/$SCRIPT_PATH" ] ; then
						SCRIPT_PATH="$DIR/$SCRIPT_PATH"
						break # only return first matching file
					fi
				done
				;;
		esac
	fi

	if [ -f "$SCRIPT_PATH" ] ; then
		DIR=`echo "$SCRIPT_PATH" | sed 's|^\(.*\)/[^/]*|\1|'` # dirname

		# echo "DEBUG: SCRIPT_PATH2=$SCRIPT_PATH"
		# echo "DEBUG: DIR=$DIR"

		if [ X"$DIR" != X ] && [ -d "$DIR" ] ; then
			# Congratulations you have the right directory
			cd "$DIR" > /dev/null 2>&1
			SCRIPT_DIR=`pwd`
			# echo "DEBUG: SCRIPT_DIR=$SCRIPT_DIR"
		else
			# we already are in the right directory, don't change
			SCRIPT_DIR=`pwd`
			# echo "DEBUG: SCRIPT_DIR=current directory ($SCRIPT_DIR)"
		fi
	else
		echo "ERROR: Unable to find directory that contains this script" 
		exit 1
	fi
}

# example: saferKill java WSA.pid
# this function takes 3 arguments
# $1 is the name of the process to be kill, it is case insensitive
# $2 is the name of the file containing the pid(s) of the process to kill,
# $3 is used to trigger logging or not. A value of "log" triggers logging 
# there can be multiple pids as long as they are separated by a space
# return 0 if it successfully killed all the processes, 1 otherwise
saferKill() {
	if [ "$#" != "3" ] ; then
		echo "ERROR: Function saferKill takes 3 arguments"
		exit 1
	fi

	PROCESS_NAME="$1"
	PID_FILE_NAME="$2"
	LOG="$3"
	RETURN_VALUE=0

	if [ -s "$PID_FILE_NAME" ]; then
		for PID in `cat $PID_FILE_NAME`; do
			# echo "DEBUG: Getting pid $PID from file $PID_FILE_NAME"
			if ps -p "$PID" 2> /dev/null | grep -i "$PROCESS_NAME" > /dev/null 2>&1 ; then
				if [ "$LOG" = "log" ] ; then echo "INFO: Attempting to stop $PROCESS_NAME process (pid=$PID), this could take a couple of seconds..."; fi
				COUNT=0
				while [ $COUNT -lt 5 ] && kill -0 "$PID" > /dev/null 2>&1 ; do
					if [ $COUNT -lt 2 ]; then
						# try to kill the process twice with a normal kill
						# echo "DEBUG: kill $PID"
						kill "$PID"
					else
						# kill did not work, try kill -9
						# echo "DEBUG: kill -9 $PID"
						kill -9 "$PID"	
					fi
					sleep 2
					COUNT=`expr $COUNT + 1`
				done

				if kill -0 "$PID" > /dev/null 2>&1; then
					RETURN_VALUE=1			
					if [ $LOG = "log" ] ; then echo "ERROR: Unable to stop $PROCESS_NAME process (pid=$PID)"; fi
				else
					if [ $LOG = "log" ] ; then echo "INFO: Successfully stopped $PROCESS_NAME process (pid=$PID)"; fi
				fi
			 # else
				# echo "DEBUG: Unable to stop $PROCESS_NAME process (pid=$PID), the process does not exist"
			fi
		done
	else
		if [ $LOG = "log" ] ; then echo "INFO: Process $PROCESS_NAME is not running" ; fi
	fi
	
	# delete the pid file
	rm -f "$PID_FILE_NAME"

	return "$RETURN_VALUE"
}

# return 0 if MQMonitor is running, 1 otherwise
isMQMonitorRunning() {
    if [ -s "$MQAGENT_PID_FILE_NAME" ]; then
		PID=`cat $MQAGENT_PID_FILE_NAME`
		if ps -p $PID 2> /dev/null | grep -i java > /dev/null 2>&1 ; then
			echo "INFO: Checking MQ Monitor java process"
			return 0
		fi
	fi
	return 1
}

startMQMonitor() {
	if isMQMonitorRunning ; then
		echo "INFO: MQMonitor is already running"
	else
	        #javaCheck
		# nohup java -cp ./lib/WebServerAgent.jar:./lib/PPWebServers.jar:./lib/commons-logging-1.1.1.jar:./lib/commons-codec-1.3.jar:./lib/jline-0.9.9.jar -Xmx512m -Dcom.wily.introscope.agentProfile=./config/WebServerAgent.profile com.wily.webservermonitor.Main  > /dev/null 2>&1 &
               nohup "$JAVA_HOME"/bin/java -Xms64m -Xmx512m -cp $CLASSPATH -Dcom.wily.introscope.agentProfile=./properties/IntroscopeAgent.profile -DProperties=./properties/ com.wily.powerpack.websphereMQ.agent.MQMonitor.MQMonitor > /dev/null 2>&1 &
    
    		echo $! > $MQAGENT_PID_FILE_NAME
			echo "INFO: MQMonitor is starting..."
			echo "INFO: To see the MQMonitor log file, execute: tail -1000lf ./logs/MQMonitor.log"
			echo "INFO: ALWAYS use MQMonitor.sh to start/stop/restart MQMonitor"
		#else
			#echo "ERROR: Unable to start WebServer PP Agent. See error messages above for details."
		#fi
		
	fi
}


stopMQMonitor() {
	if isMQMonitorRunning ; then
		echo "INFO: Stopping MQMonitor. Please wait..."
		SUCCESS="true"
		
		if saferKill java "$MQAGENT_PID_FILE_NAME" log ; then
				:
		else
				SUCCESS="false"
		fi
		
		if [ $SUCCESS = "true" ] ; then
			echo "INFO: MQMonitor was successfully stopped"
		else
			echo "ERROR: Unable to properly stopped MQMonitor. One or more processes did not stop. See error messages above for details"
		fi
	else
		echo "INFO: Either the PID file is deleted or MQMonitor is not running"
	fi
}

statusMQMonitor() {
	if isMQMonitorRunning ; then
		echo "INFO: MQMonitor is running"
	else
		echo "INFO: Either the PID file is deleted or MQMonitor is not running"
	fi
}

restartMQMonitor() {
	if isMQMonitorRunning ; then
		stopMQMonitor
		sleep 2
		startMQMonitor
	else
		echo "INFO: Either the PID file is deleted or MQMonitor is not running"
	fi
}

#javaCheck(){

#javaVar=`which java 2>&1|grep 'no java'|wc -l`

#if [ $javaVar != 0 ] ; then

#echo "Java is not available in the path. Please set a valid SUN jre path in the PATH variable"
#exit 1

#fi
#}


# main
USAGE="Usage: $0 [start|stop|restart|status]"
readonly USAGE

if [ "$#" = "1" ] ; then
	ORIGINAL_DIR=`pwd`
	readonly ORIGINAL_DIR
	if [ -L "$0" ] ; then
	var=`ls -l "$0"| awk -F">" {'print$2'}| awk '{print substr($0,2,length($0)-20)}'`
	cd "${var}" 
	else
	gotoScriptDir "$0"
	fi
    [ ! -d pids ] && /bin/mkdir pids && /bin/chmod 777 pids
    [ ! -d logs ] && /bin/mkdir logs && /bin/chmod 777 logs
	case "$1" in
		*[Rr]e[Ss]tart) restartMQMonitor ;;
		*[Ss]tart) startMQMonitor ;;
		*[Ss]top) stopMQMonitor ;;
		*[Ss]tatus) statusMQMonitor ;;
		*) echo "$USAGE" ;;
	esac

	# go back to the original directory
	cd "$ORIGINAL_DIR" > /dev/null 2>&1
else
	echo "$USAGE"
fi
